/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    timeVaryingGammaContactAngleFvPatchScalarField

Description

SourceFiles
    timeVaryingGammaContactAngleFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef timeVaryingGammaContactAngleFvPatchScalarField_H
#define timeVaryingGammaContactAngleFvPatchScalarField_H

#include "gammaContactAngleFvPatchScalarField.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class timeVaryingGammaContactAngleFvPatch Declaration
\*---------------------------------------------------------------------------*/

class timeVaryingGammaContactAngleFvPatchScalarField
:
    public gammaContactAngleFvPatchScalarField
{
    // Private data

        // Equilibrium contact angle control parameters
        scalar t0_;
        scalar thetaT0_;
        scalar te_;
        scalar thetaTe_;


public:

    //- Runtime type information
    TypeName("timeVaryingGammaContactAngle");


    // Constructors

        //- Construct from patch and internal field
        timeVaryingGammaContactAngleFvPatchScalarField
        (
            const fvPatch&,
            const scalarField&
        );

        //- Construct from patch, internal field and dictionary
        timeVaryingGammaContactAngleFvPatchScalarField
        (
            const fvPatch&,
            const scalarField&,
            const dictionary&
        );

        //- Construct by mapping given timeVaryingGammaContactAngleFvPatchScalarField
        //  onto a new patch
        timeVaryingGammaContactAngleFvPatchScalarField
        (
            const timeVaryingGammaContactAngleFvPatchScalarField&,
            const fvPatch&,
            const scalarField&,
            const fvPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchScalarField> clone() const
        {
            return tmp<fvPatchScalarField>
            (
                new timeVaryingGammaContactAngleFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        timeVaryingGammaContactAngleFvPatchScalarField
        (
            const timeVaryingGammaContactAngleFvPatchScalarField&,
            const scalarField&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchScalarField> clone(const scalarField& iF) const
        {
            return tmp<fvPatchScalarField>
            (
                new timeVaryingGammaContactAngleFvPatchScalarField(*this, iF)
            );
        }


    // Member functions

        //- Evaluate and return the time-varying equilibrium contact-angle
        virtual tmp<scalarField> theta
        (
            const fvPatchVectorField& Up,
            const fvPatchVectorField& nHat
        ) const;

        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
