/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2004 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

Application
    twoPhaseEulerFoam

Description
    Solver for a system of 2 incompressible fluid phases with one phase 
    dispersed, e.g. gas bubbles in a liquid.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "nearWallDist.H"
#include "wallFvPatch.H"
#include "Switch.H"

#include "dragModel.H"
#include "phaseModel.H"
#include "kineticTheoryModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "readEnvironmentalProperties.H"
#   include "createFields.H"
#   include "readPPProperties.H"
#   include "initContinuityErrs.H"
#   include "readTimeControls.H"
#   include "setInitialDeltaT.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    while (runTime.run())
    {
#       include "readTwoPhaseEulerFoamControls.H"
#       include "CourantNos.H"
#       include "setDeltaT.H"

        runTime++;
        Info<< "Time = " << runTime.timeName() << nl << endl;

#       include "alphaEqn.H"

#       include "liftDragCoeffs.H"

#       include "UEqns.H"

        // --- PISO loop
        for (int corr=0; corr<nCorr; corr++)
        {
#           include "pEqn.H"

            if (correctAlpha && corr<nCorr-1)
            {
#               include "alphaEqn.H"
            }
        }

#       include "DDtU.H"

#       include "kEpsilon.H"

        if (kineticTheory.on())
        {
            kineticTheory.solve();
            nuEffa += kineticTheory.mua()/rhoa;
        }

#       include "write.H"

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
