/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    sigmaRadialExtruder

Description

\*---------------------------------------------------------------------------*/

#ifndef sigmaRadialExtruder_H
#define sigmaRadialExtruder_H

#include "point.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class sigmaRadialExtruder Declaration
\*---------------------------------------------------------------------------*/

class sigmaRadialExtruder
{
    // Private data

        scalar RTbyg_;
        scalar pRef_;
        scalar pStrat_;


public:

    // Constructors

        //- Construct from components
        sigmaRadialExtruder
        (
            const scalar RTbyg,
            const scalar pRef,
            const scalar pStrat
        )
        :
            RTbyg_(RTbyg),
            pRef_(pRef),
            pStrat_(pStrat)
        {}


    // Member Operators

        point operator()
        (
            const point& surfacePoint,
            const vector& surfaceNormal,
            const label nLayers,
            const label layer
        ) const
        {
            // radius of the surface
            scalar rs = mag(surfacePoint);
            vector rsHat = surfacePoint/rs;

            scalar p = pRef_ - (layer + 1)*(pRef_ - pStrat_)/nLayers;
            scalar r = rs - RTbyg_*Foam::log(p/pRef_);

            return r*rsHat;
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
