/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2004 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    checkMesh

Description
    Checks validity of a mesh

\*---------------------------------------------------------------------------*/

#include "primitiveMesh.H"
#include "argList.H"

#include "Time.H"

#include "hexMatcher.H"
#include "wedgeMatcher.H"
#include "prismMatcher.H"
#include "pyrMatcher.H"
#include "tetWedgeMatcher.H"
#include "tetMatcher.H"

#include "regionSplit.H"
#include "cellSet.H"
#include "faceSet.H"
#include "pointSet.H"
#include "labelIOList.H"
#include "mathematicalConstants.H"
#include "parallelInfo.H"

#include "checkCoords.H"
#include "checkEdges.H"

using namespace Foam;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
#   include "addTimeOptionsNoConstant.H"
#   include "addCoordsOption.H"
#   include "setRootCase.H"
#   include "createTime.H"

    // Get times list
    instantList Times = runTime.times();

#   include "checkTimeOptionsNoConstant.H"

    runTime.setTime(Times[startTime], startTime);

#   include "createPolyMesh.H"

    bool firstCheck = true;

    for (label i=startTime; i<endTime; i++)
    {
        runTime.setTime(Times[i], i);

        Info<< "Time = " << runTime.timeName() << endl;

        polyMesh::readUpdateState state = mesh.readUpdate();

        if
        (
            firstCheck
         || state == polyMesh::TOPO_CHANGE
         || state == polyMesh::TOPO_PATCH_CHANGE
        )
        {
            firstCheck = false;

            // Clear mesh before checking
            mesh.clearOut();
            // Reconstruct parallelInfo
            mesh.parallelData();

            label noFailedChecks = 0;

#           include "checkTopo.H"
#           include "checkGeom.H"
#           include "countCellShapes.H"
#           include "countRegions.H"

            reduce(noFailedChecks, sumOp<label>());

            if (noFailedChecks == 0)
            {
                Info << "Mesh OK." << endl;
            }
            else
            {
                Info<< "Failed " << noFailedChecks << " mesh checks." << endl;
            }
        }
        else if (state == polyMesh::POINTS_MOVED)
        {
            label noFailedChecks = 0;

#           include "checkGeom.H"

            reduce(noFailedChecks, sumOp<label>());

            if (noFailedChecks == 0)
            {
                Info << "Mesh OK." << endl;
            }
            else
            {
                Info<< "Failed " << noFailedChecks << " mesh checks." << endl;
            }
        }
        else
        {
            Info << "No mesh." << endl;
        }
        Info<< endl << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
