/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description

\*---------------------------------------------------------------------------*/

#include "polyMesh.H"
#include "labelIOList.H"
#include "Cloud.H"
#include "passiveParticle.H"

using namespace Foam;

// * * * * * * * * * * * * * * * Global Functions  * * * * * * * * * * * * * //
          
void reconstructLagrangianPositions
(
    const polyMesh& mesh,
    PtrList<polyMesh>& meshes,
    const PtrList<labelIOList>& faceProcAddressing,
    const PtrList<labelIOList>& cellProcAddressing
)
{
    Cloud<passiveParticle> lagrangianPositions
    (
        mesh,
        IDLList<passiveParticle>()
    );

    forAll(meshes, i)
    {
        const labelList& cellMap = cellProcAddressing[i];

        Cloud<passiveParticle> lpi(meshes[i]);

        for
        (
            Cloud<passiveParticle>::iterator iter = lpi.begin();
            iter != lpi.end();
            ++iter
        )
        {
            const passiveParticle& ppi = iter();

            lagrangianPositions.append
            (
                new passiveParticle
                (
                    lagrangianPositions,
                    ppi.position(),
                    cellMap[ppi.cell()]
                )
            );
        }
    }
    
    lagrangianPositions.write();
}


// ************************************************************************* //
