/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    interpolatedIsoSurface

Description

SourceFiles
    interpolatedIsoSurface.C

\*---------------------------------------------------------------------------*/

#ifndef interpolatedIsoSurface_H
#define interpolatedIsoSurface_H

#include "surface.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

//class meshSearch;
class dictionary;
class plane;
class volPointInterpolation;
template<class Type> class fieldsCache;
class cellDecompIsoSurfaceCuts;

/*---------------------------------------------------------------------------*\
                           Class interpolatedIsoSurface Declaration
\*---------------------------------------------------------------------------*/

class interpolatedIsoSurface
:
    public surface
{
    // Private data

        //- Name of field to use to construct isosurface
        const word isoFieldName_;

        //- Iso surface value
        const scalar isoVal_;

        //- Generated points
        mutable pointField points_;

        //- Generated triangles
        mutable faceList faces_;

        //- Iso surface edge descriptors
        const cellDecompIsoSurfaceCuts* isoSurfCutsPtr_;

    // Private Member Functions


public:

    //- Runtime type information
    TypeName("interpolatedIsoSurface");

    // Constructors

        //- Construct from components
        interpolatedIsoSurface
        (
            const polyMesh& mesh,
            meshSearch& searchEngine,
            const word& name,
            const word& isoFieldName,
            const scalar isoVal
        );

        //- Construct from dictionary
        interpolatedIsoSurface
        (
            const polyMesh& mesh,
            meshSearch& searchEngine,
            const dictionary& dict          
        );


    // Destructor

        virtual ~interpolatedIsoSurface();


    // Member Functions

        //- Access isoSurface edge cuts
        const cellDecompIsoSurfaceCuts& isoSurfCuts() const;

        //- Points of surface
        virtual const pointField& points() const
        {
            return points_;
        };

        //- Faces of surface
        virtual const faceList& faces() const
        {
            return faces_;
        }

        //- Correct for mesh movement and/or field changes. Differs from
        //  constantIsoSurface one since meshCuts are stored for use in
        //  interpolation later on
        virtual void correct
        (
            const bool meshChanged,
            const volPointInterpolation&,
            const dictionary& interpolationSchemes,
            const fieldsCache<scalar>&,
            const fieldsCache<vector>&,
            const fieldsCache<tensor>&
        );

        //- interpolate field to surface
        virtual scalarField interpolate
        (
            const word&,
            const fieldsCache<scalar>&,
            const volPointInterpolation&,
            const dictionary& interpolationSchemes
        ) const;

        //- interpolate field to surface
        virtual vectorField interpolate
        (
            const word&,
            const fieldsCache<vector>&,
            const volPointInterpolation&,
            const dictionary& interpolationSchemes
        ) const;

        //- interpolate field to surface
        virtual tensorField interpolate
        (
            const word&,
            const fieldsCache<tensor>&,
            const volPointInterpolation&,
            const dictionary& interpolationSchemes
        ) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
